import defu from "defu";
import { unwrapTransformer } from "./transforms.mjs";
const _importShikiCore = cached(() => import("shiki/core"));
const _importShikiOptions = cached(() => import("shiki-options.mjs"));
const createCacheStore = (args) => {
  const globalCache = globalThis.__NUXT_SHIKI__ ??= {};
  const key = args[0] || "default";
  return globalCache[key] ??= {};
};
export const createHighlighter = cached(
  async () => {
    const [{ createHighlighterCore, createJavaScriptRegexEngine }, { shikiOptions }] = await Promise.all([
      _importShikiCore(),
      _importShikiOptions()
    ]);
    const highlighter = await createHighlighterCore({
      ...shikiOptions.core,
      engine: createJavaScriptRegexEngine()
    });
    highlighter.highlight = (code, highlightOptions) => {
      return highlighter.codeToHtml(code, resolveOptions(shikiOptions, highlightOptions));
    };
    return highlighter;
  },
  createCacheStore
);
export const createOptions = cached(
  async (_, nuxtApp) => {
    const { shikiOptions } = await _importShikiOptions();
    await nuxtApp.callHook("shiki:options", {
      options: shikiOptions.highlight,
      extend: (options) => {
        shikiOptions.highlight = defu(options, shikiOptions.highlight);
      }
    });
    return shikiOptions;
  },
  createCacheStore
);
export function resolveOptions(shikiOptions, highlightOptions = {}) {
  const options = defu(highlightOptions, shikiOptions.highlight);
  const unwrap = highlightOptions.unwrap ?? shikiOptions.highlight.unwrap ?? false;
  if (unwrap) {
    (options.transformers ??= []).unshift(unwrapTransformer);
  }
  return options;
}
function cached(fn, getStore) {
  const _store = getStore ? void 0 : {};
  return function(...args) {
    const store = _store || getStore(args);
    if (store.value !== void 0) {
      return store.value;
    }
    if (store.promise) {
      return store.promise;
    }
    const res = fn(...args);
    if (res instanceof Promise) {
      store.promise = res.then((value) => {
        store.value = value;
        delete store.promise;
        return value;
      });
      return store.promise;
    }
    return store.promise;
  };
}
